
namespace cpse {
namespace math {

// identity
inline void assign(basic_matrix<2, 2> &dest, identity_t)
	{ dest(0,0) = dest(1,1) = 1.f; dest(0,1) = dest(1,0) = 0.f; }

// zero
inline void assign(basic_matrix<2, 1> &d, zero_t)
	{ d.x = d.y = 0.f; }
inline void assign(basic_matrix<2, 2> &d, zero_t)
	{ d(0,0) = d(1,0) = d(0,1) = d(1,1) = 0.f; }

struct matrix2_assign_handler : matrix_assign_handler {
	using matrix_assign_handler::go;
	typedef basic_matrix<2, 2> m2;
	typedef basic_matrix<2, 1> v2;

	// inverse
	static void go(m2 &, matrix_unop<unop_inverse, m2>);

	// adjoint
	inline static void go(m2 &dest, matrix_unop<unop_adjoint, m2> src)
	{
		dest(0,0) = +src.a(1,1);
		dest(1,0) = -src.a(1,0);
		dest(0,1) = -src.a(0,1);
		dest(1,1) = +src.a(0,0);
	}

	// normalize vector
	inline static void go(v2 &dest, matrix_unop<unop_normalize, v2> src) {
		float l = sqrt(length_squared_bm(src.a));
		dest.x = src.a.x / l;
		dest.y = src.a.y / l;
	}

	// ordinary matrix multiplication
	static void go(m2 &d, matrix_binop<binop_mul, m2, m2> s);
	static void go(v2 &d, matrix_binop<binop_mul, m2, v2> s)
	{
		d.x = s.a(0,0)*s.b.x + s.a(0,1)*s.b.y;
		d.y = s.a(1,0)*s.b.x + s.a(1,1)*s.b.y;
	}
};

} // namespace math
} // namespace cpse
